%%%%%%%%%%%%%%%%%%%%%% ch10ex.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
%  This MATLAB file runs the 6 examples that appear 
%  in Chapter 10 (State-Space Design Methods) in the 
%  BookWare Companion Feedback Control Problems Using 
%  MATLAB by D. K. Frederick and J. H. Chow 
%----------------------------------------------------
clear
%  menu to select Example to be run
example = -1;	% force menu to appear
while (example < 0) | (example > 6),
  disp(' 1 <= Example 10.1: Controllability matrix')
  disp(' 2 <= Example 10.2: Pole-placement')
  disp(' 3 <= Example 10.3: Pole-placement, low gain')
  disp(' 4 <= Example 10.4: Observability matrix')
  disp(' 5 <= Example 10.5: Observer design')
  disp(' 6 <= Example 10.6: Observer-controller design')
  example = input('Select an example, or enter 0 to quit ==> ');

if example == 0, disp('Completed Chapter 10 examples'), return, end

if example == 1,
  disp(' ')
  disp('Example 10.1: Controllability matrix')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]    % state matrix
  B = [ 1; 0; 0]                      % input matrix
  disp('*****>');pause
  Co = ctrb(A,B)                      % controllability matrix
  c_eig = rank(Co)                    % number of controllable eigenvalues
  disp('*****>');pause
  disp('end of Example 10.1')
  example =-1;	% force menu to appear
end	% of example 10.1 so return to menu
%-------------------------------------

if example == 2,
  disp(' ')
  disp('Example 10.2: Pole-placement')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]  % state matrix
  B = [ 1; 0; 0]                    % input matrix
  p = [-1 -2 -3]                    % desired pole locations
  disp('*****>');pause
  F = place(A,B,p)                  % control gain
  A_cl = A-B*F                      % closed-loop system matrix
  eig(A_cl)                         % check eigenvalues
  disp('magnitude of gain')
  nrmF = norm(F)
  disp('*****>');pause
  disp('end of Example 10.2')
  example =-1;	% force menu to appear
end	% of Example 10.2 so return to menu
%-------------------------------------

if example == 3,
  disp(' ')
  disp('Example 10.3: Pole-placement, low gain')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]  % state matrix
  B = [ 1; 0; 0]                    % input matrix
  p = [-0.5 -1.14+i*1.14 -1.14-i*1.14] % desired pole locations
  disp('*****>');pause
  F = place(A,B,p)                  % control gain
  A_cl = A-B*F                      % closed-loop system matrix
  eig(A_cl)                         % check eigenvalues
  disp('magnitude of gain')
  nrmF = norm(F)
  disp('*****>');pause
  disp('end of Example 10.3')
  example =-1;	% force menu to appear
end	% of Example 10.3 so return to menu
%-------------------------------------

if example == 4,
  disp(' ')
  disp('Example 10.4: Observability matrix')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]  % state matrix
  C = [ 1 4 3.5]                      % output matrix
  disp('*****>');pause
  Ob = obsv(A,C)                    % observability matrix
  o_eig = rank(Ob)                  % number of observable eigenvalues
  disp('*****>');pause
  disp('end of Example 10.4')
  example =-1;	% force menu to appear
end	% of Example 10.4 so return to menu
%-------------------------------------

if example == 5,
  clf
  disp(' ')
  disp('Example 10.5: Observer design')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]  % state matrix
  B = [ 0; 0; 0]                    % input matrix set to zero
  C = [1 4 3.5]                     % output matrix
  D = 0                             % throughput matrix
  p = [-3 -5 -7]
  disp('*****>');pause
  L = place(A',C',p)'               % estimator gain
  A_ob = A-L*C                      % observer system matrix
  eig(A_ob)                         % check eigenvalues
  disp('*****>');pause
  xo = [1; -0.75; 0.4]                   % initial conditions
  t = [0:0.04:4];                   % time array
  u = zeros(1,101);                 % zero input
  disp('*****>');pause
  G = ss(A,B,C,D)
  [y,t,x] = lsim(G,u,t,xo);     % simulate system
  disp('Simulate Observer')
  G_ob = ss(A_ob,L,C,D)
  [y_hat,t,x_hat]=lsim(G_ob,y,t);  % simulate observer
  plot(t,x,'--',t,x_hat), grid
  title('Ex 10.5: Observer design')
  xlabel('Time (sec.)')
  ylabel('Amplitude')
  text(0.30,0.92,'x1')
  text(1.1,0.5,'x1hat')
  text(0.6,-0.12,'x2')
  text(0.9,-0.3,'x2hat')
  text(0.5,0.05,'x3')
  text(0.6,0.3,'x3hat')
  keyboard
  %
  plot(t,y,'-',t,y_hat,'--'), grid
  title('Ex 10.5: Observer design')
  xlabel('Time (sec.)')
  ylabel('Amplitude')
  text(0.25,-0.15,'y')
  text(0.55,-0.15,'yhat')
  keyboard
  disp('end of Example 10.5')
  example =-1;	% force menu to appear
end	% of Example 10.5 so return to menu
%-------------------------------------

if example == 6,
  clf
  disp(' ')
  disp('Example 10.6: Observer-controller design')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0]       % state matrix
  B = [ 1; 0; 0]                         % input matrix set to zero
  C = [1 4 3.5]                          % output matrix
  D = 0                                  % throughput matrix
  Gp = ss(A,B,C,D)
  disp('*****>');pause
  p_s = [-1 -2 -3] % desired system poles 
  F = place(A,B,p_s)                     % control gain
  disp('*****>');pause
  p_o = [-3 -5 -7]
  L = place(A',C',p_o)'                  % estimator gain
  disp('*****>');pause
  A_f = A-B*F-L*C                        % controller system matrix
  f_poles = eig(A_f)                     % controller poles
  f_zeros = tzero(A_f,-L,-F,0)           % controller zeros
  disp('*****>');pause
  Gc = ss(A_f,-L,-F,0)
  bode(Gc)                               % frequency response plot
  title('Ex 10.6: Frequency response of controller') 
  keyboard
  GcGp = Gc*Gp                           % cascade system
  T = feedback(GcGp,1,-1)                % closed-loop system
  cl_loop_poles = pole(T)               % closed-loop system poles
  disp('*****>');pause
  subplot(1,1,1)
  lfg = dcgain(T)
  N = 1/lfg
  T_ref = N*T;
  z0 = [0 0 0 1 -0.75 0.4]'
  t = [0:0.02:4]';
  u = 0*t;
  [y,t,x] = lsim(T_ref,u,t,z0);
  plot(t,x(:,1:3),t,x(:,4:6),'--'), grid
  title('Ex 10.6: Initial condition response')
  text(0.23,0.85,'x1')
  text(0.9,0.4,'x1hat')
  text(0.55,-0.1,'x2')
  text(0.9,-0.3,'x2hat')
  text(0.6,0.05,'x3')
  text(1.4,0.18,'x3hat')
  keyboard
  step(T_ref), grid
  title('Ex 10.6: Step input response')
  [ys,t,x] = step(T_ref);
  [Mo,tp,tr,ts,ess] = tstats(t,ys,1.0)
  keyboard
  disp('end of Example 10.6')
  example =-1;	% force menu to appear
end	% of Example 10.6 so return to menu
%-------------------------------------
end		% of while
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
