%%%%%%%%%%%%%%%%%%%%%% cp10_3.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%	     Comprehensive Problem CP10.3
%	does observer-controller design for
%	rigid stick balancer, using both outputs
%		y1 = theta, y2 = x
%	states are: theta, th_dot, x, v
%	Has 2 reference inputs: theta_des & x_des
%----------------------------------------------------
disp('CP10.3: Observer-controller design for rigid stick balancer')
clear;clf
% load rigid.mat if exists, otherwise quit this M-file
if chkmat('rigid') == 1; 
    load rigid.mat
  else
    return
end
%---- plant matrices, 1 input, 2 outputs
Ap = A;
Bp = B;
disp('******>'),pause
Cp = C;
Dp = D;
Gp = ss(Ap,Bp,Cp,Dp)

disp('******>'),pause
% 
disp('cp10_3: observer-controller design for')
disp('.....rigid stick balancer, using both outputs')
disp('.....y1 = theta, y2 = x')
%
%%%pp = [-1; -2; -3; -4]
pp = input('enter 4 desired CL eigenvalues as column ==> ')
Rpp_str = vec2str(real(pp));
Ipp_str = vec2str(imag(pp));
F = place(Ap,Bp,pp)
disp('******>'),pause
L = place(Ap',Cp',2*pp)'
disp('******>'),pause

Aoc = Ap-Bp*F-L*Cp	% observer-controller (4 x 4)
Boc = L			%     (4x2)
disp('******>'),pause
Coc = -F		%     (1x4)
Doc = zeros(1,2)	%     (1x2)
Goc = ss(Aoc,Boc,Coc,Doc);

disp('******>'),pause
pOC=eig(Aoc)		% obsvr-ctrlr poles
disp('******>'),pause
zOc=tzero(Goc)	% obsv-ctrlr zeros
disp('******>'),pause
%--- series connection
Gocp = Goc*Gp;
[Aocp,Bocp,Cocp,Docp] = ssdata(Gocp); % 1 input,2 outputs
%---- unity fdbk connection using both theta & cart psn
%     positive sign at SJ
Gcl = feedback(Gocp,1);
[Acl,Bcl,Ccl,Dcl] = ssdata(Gocp);
%
%======= unit step in desired cart psn =======
t=[0:0.05:10]';
x_ref = -ones(size(t));
for i = find(t<1)',
   x_ref(i) = 0;
end
theta_ref = 0*t;
ref = [theta_ref x_ref];
[y,x] = lsim(Gcl,ref,t);
subplot(2,1,1)
plot(t,y(:,1));grid
title('CP10.3: rigid stick; obsv-ctrlr using both theta & x')
xlabel('time (secs)')
ylabel('stick angle (deg)')
text('Units','norm','Pos',[0.5 0.90],'Str','Controller poles')
text('Units','norm','Pos',[0.5 0.75],...
		'Str',['real parts = ' Rpp_str])
text('Units','norm','Pos',[0.5 0.60],'Str',...
		['imag parts = ' Ipp_str])
subplot(2,1,2)
plot(t,y(:,2),t,-x_ref,'--');grid
ylabel('cart psn (ft)')
xlabel('time (secs)')
text('Units','norm','Pos',[0.2 0.20],'Str',...
		'Observer poles are 2 * controller poles')
keyboard
%
%====== IC response: 1 deg on stick, rest = 0 =========
%.......stick angle is state #5 
x0 = [zeros(4,1); 1; zeros(3,1)];	
[y,x] = lsim(Gcl,[0*t 0*t],t,x0);
subplot(2,1,1)
plot(t,y(:,1));grid
title('CP10.3: rigid stick; initial stick angle, x_ref = 0')
xlabel('time (secs)')
ylabel('stick angle (deg)')
text('Units','norm','Pos',[0.5 0.90],...
			'Str','Controller poles')
text('Units','norm','Pos',[0.5 0.75],'Str',...
			['real parts = ' Rpp_str])
text('Units','norm','Pos',[0.5 0.60],'Str',...
			['imag parts = ' Ipp_str])
subplot(2,1,2)
plot(t,y(:,2));grid
ylabel('cart psn (ft)')
xlabel('time (secs)')
text('Units','norm','Pos',[0.2 0.80],'Str',...
		'Observer poles are 2 * controller poles')
keyboard
disp('end of Comprehensive Problem 10.3')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
