%%%%%%%%%%%%%%%%%%%%%% cp4_3.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 5/28/99 HS
%----------------------------------------------------
%        Comprehensive Problem CP4.3:
%   Open-loop rigid stick with no inputs,
%   but with positive initial stick angle
%   and negative initial stick angular velocity.
%   Use trial and error to find omega(0) value
%   such that stick comes to vertical and stays
%   close for a long time, before falling back
%   or going over the top.
%   The following initial state vector will
%   do the job:   x0 = [0.1 -0.28232 0 0]
%----------------------------------------------------
disp('CP4.3: Open-loop rigid stick')
clear;clf
% load rigid.mat if exists, otherwise quit this M-file
if chkmat('rigid') == 1, 
    load rigid.mat
  else
     return
end
disp('Try to find initial angular velocity that will')
disp('   keep the stick from falling over for 4 seconds')
A
B
disp('*******>'),pause
C
D
disp('*******>'),pause
Gss = ss(A,B,C,D);
disp('eigenvalues of A')
eigA=eig(A)
disp('*******>'),pause
theta0 = 0.1;	% initial stick angle in radians
omega0 = input('Enter initial angular velocity (< 0) ==>');
omega0_str = num2str(omega0);
x0 = [theta0  omega0  0  0];
tmax = input('Enter maximum time ==>');
t = [0:0.01:tmax]';
u = 0*t;	% zero input
[y,x]=lsim(Gss,u,t,x0);
theta = y(:,1);
x_cart = y(:,2);
subplot(2,1,1)
plot(t,theta);grid
title('CP4.3: OL rigid stick with zero input')
xlabel('Time (secs)'); ylabel('angle (rad)')
text('Units','norm','Pos',[0.1,0.8],'Str',...
	['initial angular velocity = ' omega0_str ' rad/s'])
subplot(212)
plot(t,x_cart);grid
xlabel('Time (secs)'); ylabel('position (ft)')
subplot(111)
keyboard
%------- transformation to modal form
disp('model in modal form is:')
disp('')
Gm = canon(Gss,'modal')
disp('******>'),pause
disp('redisplay the A matrix will all entries having')
disp('....magnitudes less than 10^(-9) set to zero')
[Am,Bm,Cm,Dm] = ssdata(Gm);
AA = small20(Am,1.e-9)
disp('You should see the eigenvalues on the main diagonal')
disp('end of Comprehensive Problem 4.3')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%