%%%%%%%%%%%%%%%%%%%%%% cp4_4.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 5/28/99 HS
%----------------------------------------------------
%         Comprehensive Problem CP4.4:
%   Open-loop flexible stick with no inputs,
%   but with positive initial stick angle
%   and negative initial stick angular velocity.
%   Use trial and error to find omega(0) value
%   such that stick comes to vertical and stays
%   close for a long time, before falling back
%   or going over the top.
%   The following initial state vector will
%   do the job:   x0 = [0.1 -0.28232 0 0]
%----------------------------------------------------
disp('CP4.4: Open-loop flexible stick')
clear;clf
% load flex.mat if exists, otherwise quit this M-file
if chkmat('flex') == 1, 
    load flex.mat
  else
     return
end
disp('Stick balancer with flexible stick')
disp('Try to find initial angular velocity that will')
disp('   keep the stick from falling over for 4 seconds')
A
B
disp('*******>'),pause
C
D
disp('*******>'),pause
G = ss(A,B,C,D);
disp('eigenvalues of A')
eigA=eig(A)
disp('*******>'),pause
theta0 = 0.1;	% initial stick angle in radians
omega0 = input('Enter initial angular velocity (< 0) ==>');
omega0_str = num2str(omega0);
x0 = [theta0  omega0  0  0  0  0];	% 6 states
tmax = input('Enter maximum time ==>');
t = [0:0.01:tmax]';
u = 0*t;	% zero input
[y,x]=lsim(G,u,t,x0);
theta = y(:,1);
x_cart = y(:,2);
subplot(211)
plot(t,theta);grid
title('CP4.4: OL flexible stick with zero input')
xlabel('Time (secs)'); ylabel('angle (rad)')
text('Units','norm','Pos',[0.1,0.8],'Str',...
	['initial angular velocity = ' omega0_str ' rad/s'])
subplot(212)
plot(t,x_cart);grid
xlabel('Time (secs)'); ylabel('position (ft)')
subplot(111)
keyboard
%------- transformation to modal form
disp('model in modal form is:')
disp('')
Gm = canon(G,'modal')
[Am,Bm,Cm,Dm] = ssdata(Gm)
disp('******>'),pause
disp('redisplay the A matrix will all entries having')
disp('....magnitudes less than 10^(-9) set to zero')
AA = small20(Am,1.e-9)
disp('You should see the eigenvalues on the main diagonal')
disp('....except for the lower right 2 x 2 block which')
disp('....is indicating a pair of imaginary eigenvalues')
disp('....at s = +j20 and s = -j20')
disp('end of Comprehensive Problem 4.4')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

