%%%%%%%%%%%%%%%%%%%%%% ex8_3.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
%	             PID control
%----------------------------------------------------
clear,clf
disp('Example 8.3: PID control')
yes = 'y'; no = 'n'; 
repeat = yes;
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
numGp = Kproc;
denGp = conv([tauP1 1],[tauP2 1]);
H     =tf(1,[tau_sen 1]);
%
%============== Part (a): sweep KP for various KD values
disp('sweep KP for various KD values')
disp('setting KI = 0.5 based on PI design')
KI = 0.5;
KI_str=num2str(KI);
denGc = [1 0];
denG = conv(denGc,denGp);
%----- CL step response to reference input
t = [0:0.01:2]';
while(repeat == yes),	% allow multiple runs
  disp(' ')
  disp('Book uses KD = 0, 0.1, 0.2, 0.3, 0.4 & 0.5')
  disp(' ')
  KD = input('enter derivative gain KD ==> ');
  disp(' ')
  KD_str=num2str(KD);
  for KP = [1:1:10];
    numKc = KP*[KD 1 KI];   % numerator of PID controller
    numGK = conv(numKc,numGp);
    GK = tf(numGK,[denGp 0])
    T = feedback(GK,H,-1);
    ys = step(T,t);
    plot(t,ys)
    if ishold ~=  1, hold on, end
  end	% of 'for'
  xlabel('Time (sec.)'),ylabel('Amplitude')
  grid;
  title('Ex 8.3: CL step resp, PID control, Fig. 8.6')
  text(1.0,0.35,'KP = 1,2,3,...,10')
  text(1.0,0.25,['KI = ' KI_str])
  text(1.0,0.15,['KD = ' KD_str])
  text(0.25,1.16,'KP = 10')
  text(0.7,0.65,'KP = 1')
  hold off
  keyboard
  reply = input('repeat for another value of KD(yes/no)? ==> ')
  if reply == yes, repeat = yes;
   else repeat = no;
  end	% of 'if'
end	% of 'while' 
%
%=============== Part (b): root locus =============
disp('Part (b): root locus')
disp('setting KD = 0.4')
KD = 0.4;
KD_str=num2str(KD);
%-- Note: can not use series cmd because Gc(s) is not proper
%   Also, can not use: numRL = conv([KD 1 KI],numGpH);
%      to form numerator because  numGpH has been padded
%      with 3 leading zeros, causing a bogus error message
%      from the MATLAB routine tfchk
    numKc = KP*[KD 1 KI];   % numerator of PID controller
    numGK = conv(numKc,numGp);
    GK = tf(numGK,[denGp 0])
    disp('draw root locus with default area')
    rlocus(GK*H); grid
axis equal
title('Ex 8.3: open-loop, PID control')
text(20,-10,['KI = ' KI_str])
text(20,-15,['KD = ' KD_str])
keyboard
disp('redraw locus with smaller area')
axis([-15 5 -10 10])
title('Ex 8.3: open-loop, PID control, Fig. 8.7(a)')
text(0,-5,['KI = ' KI_str])
text(0,-6,['KD = ' KD_str])
keyboard
disp('redraw locus with still smaller area')
axis([-2.5 0.5 -1.2 1.2])
title('Ex 8.3: open-loop, PID control, Fig. 8.7(b)')
text(-0.2,-0.85,['KI = ' KI_str])
text(-0.2,-1.0,['KD = ' KD_str])
keyboard
[kk,polesCL] = rlocfind(GK*H)

%==== Part (c): reference & disturbance step responses ===
disp('Part (c): reference & disturbance step responses')
disp('setting KP = 0.6')
KP = 7.0;	% selected for 10% overshoot
KP_str=num2str(KP);
numKc = KP*[KD 1 KI];   % numerator of PID controller
numGK = conv(numKc,numGp);
GK = tf(numGK,[denGp 0])
%----- closed-loop step response to reference input
T_ref = feedback(GK,H,-1);
t = [0:0.01:3];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
Gp = tf(numGp,denGp)
HK = tf(-numKc,[tau_sen 1 0]);
%-------- CL step response to disturbance input
%--- use +1 for feedback gain because neg sign is in HK(s)
T_dist = feedback(Gp,HK,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 8.3: CL step responses, PID control, Fig. 8.8')
text(2.0,0.50,['KP = ' KP_str])
text(2.0,0.40,['KI = ' KI_str])
text(2.0,0.30,['KD = ' KD_str])
text(1.0,1.05,'reference')
text(1.0,0.15,'disturbance')
disp('end of Example Problem 8.3')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
