%%%%%%%%%%%%%%%% fig8_8.m %%%%%%%%%%%%%%%
%
%      2nd order plant with 2 real poles with PID control
% 	runs reference & disturbance step response for
%  	Jan 99
%   last revision:  1/6/99  JHC
%----------------------------------------
clear; clf
set(gcf,'DefaultAxesFontSize',16)
set(gcf,'DefaultTextFontSize',16)
disp('fig 8.8: PID control')
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
numGp = Kproc;
denGp = conv([tauP1 1],[tauP2 1]);
H     =tf(1,[tau_sen 1]);
KP = 7;
KI = 0.5;
KD = 0.4;
KP_str=num2str(KP);
KI_str=num2str(KI);
KD_str=num2str(KD);
numKc = KP*[KD 1 KI];   % numerator of PID controller
numGK = conv(numKc,numGp);
GK = tf(numGK,[denGp 0])
%----- closed-loop step response to reference input
T_ref = feedback(GK,H,-1);
t = [0:0.01:3];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
Gp = tf(numGp,denGp)
HK = tf(-numKc,[tau_sen 1 0]);
%-------- CL step response to disturbance input
%--- use +1 for feedback gain because neg sign is in HK(s)
T_dist = feedback(Gp,HK,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
%title('fig 8.8: CL responses to ref & dist steps, PID control')
text(2.0,0.50,['KP = ' KP_str])
text(2.0,0.40,['KI = ' KI_str])
text(2.0,0.30,['KD = ' KD_str])
text(1.0,1.05,'reference')
text(1.0,0.15,'disturbance')
axis([0 3 0 1.2])
print -deps fig8_8.eps
%-- reset fonts to defaults
set(gcf,'DefaultAxesFontSize',get(0,'DefaultAxesFontSize'));
set(gcf,'DefaultTextFontSize',get(0,'DefaultTextFontSize'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%