%%%%%%%%%%%%%%%%%%%%%% cp9_1b.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%  M-file for designing lag controller for the power 
%    system example for Comprehensive Problem CP9.1b
%----------------------------------------------------
clear,clf
disp('Comprehensive Problem CP9.1b')

epow    % read data

C = C(1,:);
cflag = input('enter 1 to add more damping to system ==> ')
if cflag == 1
  A(2,2) = 10*A(2,2);
end

disp('Gain and phase margins, from V_field to V_term')
Gpss = ss(A,B,C,0);
Gp = tf(Gpss);
margin(Gp)
disp('*****>'), pause

disp('Lag control via Kuo''s algorithm')

ess_des = input('Enter desired steady-state error ==> ')
Kp_des = (1/ess_des) - 1
lfg = dcgain(Gp)
Klag = Kp_des/lfg
disp('*********>');pause

%---------- open-loop transfer function
KGp = Klag*Gp;
margin(KGp)
w = logspace(-2,3,100)';
[mag_ratio_K,ph_K] = bode(KGp,w);
mag_db_K = 20*log10(mag_ratio_K);
disp('*********>');pause

pm_des = input('enter desired phase margin in degrees ==>')
pm_act = pm_des + 10;  % add 10 deg to desired margin for lag design
phase_0 = ph_K - pm_act + 180;    % = 0 if matches pm_act
[ph,i] = min(abs(phase_0));       % best match
Alag = mag_ratio_K(i)           % alpha
Zlag = -w(i)/10
%------------- build the lag then use
%		forward transfer function is Gc * Gp
numGc = Klag*[1/(-Zlag) 1];
denGc = [Alag/(-Zlag) 1];
Gc = tf(numGc,denGc);
Gcp = Gc*Gp;
[gm,pm,wgm,wpm] = margin(Gcp);
disp('gain margin in dB & omega_gm are:')
disp([20*log10(gm)  wgm])
disp('phase margin in degrees & omega_pm are:')
disp([pm wpm])
disp('*********>');pause
%---- do the plot
margin(Gcp)
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(Gcp)

%------------- closed-loop step response to reference input
Gcp_CL = feedback(Gcp,1);
z=tzero(Gcp_CL)
p=pole(Gcp_CL)
t = [0:0.02:10];
step(Gcp_CL,t)
grid; title('CP9.1b: Closed loop step response')
keyboard
[mag_ratio_lag,ph_lag] = bode(Gcp,w);
mag_db_lag = 20*log10(mag_ratio_lag);
%--------- now plot both sets
zdb = 0*w;
subplot(2,1,1)
semilogx(w,mag_db_lag(:),'-',w,mag_db_K(:),'--',w,zdb,':');
grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
title('CP9.1b: Lag controller (solid); Proportional controller (- -)')
subplot(2,1,2)
ph180 = 0*w-180;
semilogx(w,ph_lag(:),'-',w,ph_K(:),'--',w,ph180,':');
grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
disp('end of Comprehensive Problem 9.1b')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
