% Model file power.m
% This M-file contains the model of the single-machine 
% infinite-bus system described in Appendix A of "Feedback 
% Control Problems using MATLAB and the Control System 
% Toolbox" by D. K. Frederick and J. H. Chow. The system 
% has 7 states, 1 input and 3 outputs. 
%
% See also: powerxd

%%%%%%%%%%%%%%%%%%%%%% power.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB 
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%----------------------------------------------------
A = [       0   3.7699e+02            0            0 ...
            0            0            0;
  -2.4617e-01  -1.5633e-01  -1.3687e-01  -1.2318e-01 ...
  -1.2401e-02  -5.4566e-02            0;
   1.0922e-01   2.6271e-01  -2.1671e+00   2.3009e+00 ...
  -1.7068e-02  -7.5289e-02   1.2715e+00;
  -4.5840e+00            0   2.9954e+01  -3.4332e+01 ...
            0            0            0;
  -1.6060e-01            0            0            0 ...
  -8.4387e+00   6.3263e+00            0;
  -1.7004e+00            0            0            0 ...
   1.5222e+01  -2.1546e+01            0;
  -3.3926e+01  -2.3137e+01   6.8627e+00  -5.9469e+01 ...
   1.5038e+00   6.6335e+00  -1.1439e+02];

B = [ 0; 0; 0; 0; 0; 0; 16.4];

C = [  -0.1226    1.0500    0.2304    0.2074  ...
       -0.1045   -0.4599         0;
          0         1         0         0     ...
          0         0            0; 
	1.4158    0.9000    0.7871    0.7084  ...
        0.0713    0.3138   -0.0000];

D = zeros(3,1);

C1 = C(1,:); C2 = C(2,:); C3 = C(3,:);
[num1,den] = ss2tf(A,B,C1,0);
num1 = small20(num1,1e-10);
[num2,den] = ss2tf(A,B,C2,0);
num2 = small20(num2,1e-10);
[num3,den] = ss2tf(A,B,C3,0);
num3 = small20(num3,1e-10);

save power.mat A B C D C1 C2 C3 num1 num2 num3 den 

disp('Power system state-space model in arrays A B C D')
disp('  outputs are Vterm, w(speed), P')
disp('Power system transfer function')
disp('  den is denominator')
disp('  num1 is numerator from u to Vterm')
disp('  num2 is numerator from u to w(speed)')
disp('  num3 is numerator from u to P')
disp('The system data is saved in power.mat')

%%%%%%%%%%%%%%%%%% end of power.m %%%%%%%%%%%%%%%%%%%
